/*
 * Copyright (c) 2015 Dennis Clark TTT Enterprises
 * You are free to do as you please with it, ask long as you mention you got it from me.
 * Also, I like craft Porters or Stouts, as well as a good Shiraz.
 * In case you might be wondering, that is...
 * 
 * PICd36T
 * 
 * Shows the use of the 4D Systems uCAM II interfaced to a 4D Systems PICadillo 35T display.
 * 
 */

#include <DSPI.h>
#include <TFT.h>

// Configure the display
PICadillo35t tft;

// camera screen "pages"
uint16_t pg1[16384];

// array pointers
uint8_t junk;

// changing bytes into words for graphics
union {
	uint16_t word;
	uint8_t byte[2];
} convert;

// Command and status arrays
uint8_t cmd[6];
uint8_t status[6];
uint8_t status2[6];

// Commands
uint8_t INIT[6] = {0xAA,0x01,0x00,0x06,0x09,0x00};		// 16bit 565RGB color 128x128 raw format
uint8_t GETPIC[6] = {0xAA,0x04,0x02,0x00,0x00,0x00};	// GET PICTURE Raw picture mode
uint8_t RESET[6] = {0xAA,0x08,0x01,0x00,0x00,0x00};		// RESET state machines only
uint8_t HRESET[6] = {0xAA,0x08,0x01,0x00,0x00,0xFF};	// RESET whole camera
uint8_t SYNC[6] = {0xAA,0x0D,0x00,0x00,0x00,0x00};		// wake the camera up
uint8_t ACKF[6] = {0xAA,0x0E,0x0A,0x00,0x00,0x00};		// ACK a complete frame received
uint8_t ACKS[6] = {0xAA,0x0E,0x0D,0x00,0x00,0x00};		// ACK the SYNC response
uint8_t BAUD[6] = {0xAA,0x07,0x00,0x00,0x00,0x00};		// Set BAUD to 3686400

// camera responses to pay attention to in status second byte
#define CDATA 0x0A										// Data here giving type and length of file coming
#define CSYNC 0x0D
#define CACK 0x0E										// Everything fine, byte 3 has the command ID being ACK'd
#define CNACK 0x0F										// something bad happened, error in 3rd byte

void printCentered(uint16_t pos, char *data) {
	uint16_t w = tft.stringWidth(data);
	tft.setCursor((tft.getWidth() / 2) - (w / 2), pos);
	tft.print(data);
}

void setup() {
	uint16_t n;
	uint16_t d;
	Serial.begin(9600);


	// Set up Picadillo display hardware.
	analogWrite(PIN_BACKLIGHT, 255);
	tft.initializeDevice();
	tft.setRotation(0);
	tft.fillScreen(Color::Black);
	tft.setFont(Fonts::Topaz);
	tft.setTextColor(Color::White, Color::Black);	
	printCentered(50, "uCAM II 128x128 RGB565 image");
	printCentered(70, "Camera Operation Demonstration");

  	// Talk to the camera, this is the max auto-baud detect that will work.
  	Serial1.begin(921600);		

  	Serial.println("Wake up the camera...");
  	status[1] = 0;
  	junk = 0;
  	Serial.print("try 60 times ");
  	
  	d=4;
  	while (status[1] != CACK) {
  		Serial1.write(SYNC,6);
  		if (Serial1.available() ) {
  			for (n=0;n<6;n++) {
  				while (!Serial1.available());
	  			status[n] = Serial1.read();
  			}
  			
  			while(!Serial1.available());
  			for (n=0;n<6;n++) {
  				while (!Serial1.available());
  				status2[n] = Serial1.read();
  			}
  			Serial1.write(ACKS,6);					// ACK the successful SYNC
  			break;
  		}
  		junk = junk+1;
  		if (junk > 60) {
  			Serial.println("Unable to sync.");
  			while(1);
  		}
  		d+=1;
  		delay(d);
  	}
	
  	Serial.print("Error: ");Serial.println((int)status[4],HEX);;

	// In case we got a NACK for a SYNC when the camera was awake, flush the queue out
	while (Serial1.available()) {
		Serial1.read();
	}
  	// Set up to get pictures "streamed"
  	Serial.println("Send INIT");
  	Serial1.write(INIT,6);							// INIT for a small raw 565RGB screen
  	while (!Serial1.available());					// wait for the ACK
  	for (n=0;n<6;n++) {
  		while(!Serial1.available());
  		status[n] = Serial1.read();
  	}
  	if (status[1] != CACK) {						// ACK, that command passed
  		Serial.println("We barfed on INIT.");
  		  	
 	 	Serial.print("Error: ");Serial.println((int)status[4],HEX);

  		while(1);									// freeze here
  	}

 	 //Give the camera time to adjust
 	 delay(2000);
	
}

void loop() {
	static uint32_t ts = millis();
	static uint32_t count = 0;
	uint16_t n;

	// Lets get a picture!  I hope...
	Serial1.write(GETPIC,6);						// Send the GET PICTURE command
  	while (!Serial1.available());					// wait for the ACK
  	
  	for (n=0;n<6;n++) {
  		while(!Serial1.available());
  		status[n] = Serial1.read();
  	}
  	if (status[1] != CACK) {						// ACK, that command passed
  		Serial.println("We barfed on GET PICTURE.");
  		  	
 	 	Serial.print("Error: ");Serial.println((int)status[4],HEX);
  		while(1);									// freeze here
  	}

  	for (n=0;n<6;n++) {
  		while(!Serial1.available());
  		status[n] = Serial1.read();
  	}
  	if (status[1] != CDATA) {						// We got the data response
  		Serial.println("We barfed on the DATA return.");
  		  	
 	 	Serial.print("Error: ");Serial.println((int)status[4],HEX);
  		while(1);									// freeze here
  	}
  	// We aren't going to look at the image size, we know what it is already.
	
	while (!Serial1.available());					// wait for DATA response
  	for (n=0;n<16384;n++) {							// Get our screen image
  		while(!Serial1.available());
  		convert.byte[1] = Serial1.read();
  		while(!Serial1.available());
  		convert.byte[0] = Serial1.read();
  		pg1[n] = convert.word;						// just grab data as fast as we can!
  	}
  	
  	delay(1);
	Serial1.write(ACKF,6);							// ACK that we got the image



	// send data to the "window" on the Picadillo 35T
	tft.openWindow(96, 128, 128, 128);
	tft.windowData(pg1,16384);
	tft.closeWindow();
	
	count++;
	if (millis() - ts >= 10000) {
		float fps = count / 10.0;
		tft.setCursor(120, 400);
		tft.print(fps);
		tft.print(" fps");
		count = 0;
		ts = millis();
	}
}
